@@ -1,25 +1,22 @@
-import argparse
 import ast
+import ast
+import base64
 import codecs
+import csv
 import getpass
 import importlib
+import json
 import logging
 import os
-import sys
-import csv
 import re
-from enum import Enum
-
-import lxml
-import pandas as pd
-import pyvis
-from lxml import html
-from pathlib import Path
+import sys
 import time
-import urllib3
+from enum import Enum
+from pathlib import Path
+
+import pandas as pd
 import requests
-import json
-import base64
+
 
 # THIS FILE HAS NO amilib IMPORTS TO AVOID CYCLIC DEPENDENCIES
 def _get_logger(name,
@@ -55,13 +52,14 @@ def _get_logger(name,
     err_level = logging.ERROR
     errHandler.setLevel(err_level)
     # Create a log format using Log Record attributes
-    format = "%(name)s: %(asctime)s | %(levelname)s | %(filename)s:%(lineno)s | %(process)d >>> %(message)s"
-    format = "%(levelname)s %(filename)s:%(lineno)s:%(message)s"
-    fmt = logging.Formatter(format)
+    formatx = "%(name)s: %(asctime)s | %(levelname)s | %(filename)s:%(lineno)s | %(process)d >>> %(message)s"
+    formatx = "%(levelname)s %(filename)s:%(lineno)s:%(message)s"
+    fmt = logging.Formatter(formatx)
     # Set the log format on each handler
     stdoutHandler.setFormatter(fmt)
     errHandler.setFormatter(fmt)
     # Add each handler to the Logger object
+    # I get 2 logger message every time - not sure why
     logger.addHandler(stdoutHandler)
     logger.addHandler(errHandler)
     return logger
@@ -75,21 +73,18 @@ HREF = "href"
 class Util:
     """Utilities, mainly staticmethod or classmethod and not tightly linked to AMI"""
 
-
     @classmethod
     def get_logger(cls,
-                    name,
-                    level=logging.DEBUG,
-                    err_level=logging.ERROR,
-                    err_log="error.log",
-                    format="%(name)s | %(levelname)s | %(filename)s:%(lineno)s |>>> %(message)s",
-                    ):
+                   name,
+                   level=logging.DEBUG,
+                   err_level=logging.ERROR,
+                   err_log="error.log",
+                   formatx="%(name)s | %(levelname)s | %(filename)s:%(lineno)s |>>> %(message)s",
+                   ):
         """
         creates logger with name == __name__
         """
-        return _get_logger(name, level, err_level, err_log, format)
-
-
+        return _get_logger(name, level, err_level, err_log, formatx)
 
     @classmethod
     def set_logger(cls, module,
@@ -149,6 +144,11 @@ class Util:
 
     @classmethod
     def read_pydict_from_json(cls, file):
+        """
+        read json file into python dictionary
+        :param file: to read
+        :return: Python dict
+        """
         with open(file, "r") as f:
             contents = f.read()
             dictionary = ast.literal_eval(contents)
@@ -203,12 +203,14 @@ class Util:
         sys.argv.extend(args)
 
     @classmethod
-    def create_name_value(cls, arg: str, delim: str = "=") -> tuple:
+    # def create_name_value(cls, arg: str, delim: str = "=") -> tuple | None:
+    def create_name_value(cls, arg: str, delim: str = "="):
+
         """create name-value from argument
         if arg is simple string, set value to True
         if arg contains delimeter (e.g. "=") split at that
         :param arg: argument (with 0 or 1 delimiters
-        :param delim: delimiter (default "=", cannot be whitespace
+        :paramhitory delim: delimiter (default "=", cannot be whitespace
         :return: name, value , or name, True or None
         """
         if not arg:
@@ -320,7 +322,7 @@ class Util:
     def get_urls_from_webpage(cls, suffixes, weburl):
 
         page = requests.get(weburl)
-        tree = html.fromstring(page.content)
+        tree = lxml.html.fromstring(page.content)
         ahrefs = tree.xpath(f".//a[@{HREF}]")
         urls = []
         for sf in suffixes:
@@ -428,37 +430,39 @@ class Util:
             logger.debug(f"not b64: {s}")
             return False
 
-    @classmethod
-    def create_pyviz_graph(cls, incsv, anchor="anchor", target="target", outpath=None):
-        """creates network graph from CSV file
-        :param incsv: csv filename
-        :param anchor: name of anchor column (def 'anchor')
-        :param target: name of target column (def 'target')
-        :param outpath: file to draw graph to (def None)
-        uses pyvis_graph.force_atlas_2based() for layout (will give moer options later
-        """
-        try:
-            with open(str(incsv), "r") as f:
-                data = pd.read_csv(f)
-        except Exception as e:
-            logger.error(f"cannot read {incsv} because {e}")
-            return
-        anchors = cls.get_column(data, anchor, incsv)
-        targets = cls.get_column(data, target, incsv)
-        if anchors is None or targets is None:
-            logger.error(f"Cannot find anchors/targets in CSV {incsv}")
-            return None
-        pyvis_graph = pyvis.network.Network(notebook=True)
-        for a, t in zip(anchors, targets):
-            pyvis_graph.add_node(a, label=a)  # also color, size
-            pyvis_graph.add_node(t, label=t)
-            pyvis_graph.add_edge(a, t)  # also color
-        pyvis_graph.force_atlas_2based()
-        if outpath:
-            try:
-                pyvis_graph.show(str(outpath))
-            except Exception as e:
-                logger.error(f"Cannot write pyviz graph to {outpath} because {e}")
+    # removed pyvis - not used
+    # @classmethod
+    # def create_pyviz_graph(cls, incsv, anchor="anchor", target="target", outpath=None):
+    #     """creates network graph from CSV file
+    #     :param incsv: csv filename
+    #     :param anchor: name of anchor column (def 'anchor')
+    #     :param target: name of target column (def 'target')
+    #     :param outpath: file to draw graph to (def None)
+    #     uses pyvis_graph.force_atlas_2based() for layout (will give moer options later
+    #     """
+    #     import pyvis
+    #     try:
+    #         with open(str(incsv), "r") as f:
+    #             data = pd.read_csv(f)
+    #     except Exception as e:
+    #         logger.error(f"cannot read {incsv} because {e}")
+    #         return
+    #     anchors = cls.get_column(data, anchor, incsv)
+    #     targets = cls.get_column(data, target, incsv)
+    #     if anchors is None or targets is None:
+    #         logger.error(f"Cannot find anchors/targets in CSV {incsv}")
+    #         return None
+    #     pyvis_graph = pyvis.network.Network(notebook=True)
+    #     for a, t in zip(anchors, targets):
+    #         pyvis_graph.add_node(a, label=a)  # also color, size
+    #         pyvis_graph.add_node(t, label=t)
+    #         pyvis_graph.add_edge(a, t)  # also color
+    #     pyvis_graph.force_atlas_2based()
+    #     if outpath:
+    #         try:
+    #             pyvis_graph.show(str(outpath))
+    #         except Exception as e:
+    #             logger.error(f"Cannot write pyviz graph to {outpath} because {e}")
 
     @classmethod
     def get_column(cls, data, colname, csvname=None):
@@ -572,11 +576,36 @@ class Util:
     @classmethod
     def normalize_chars(cls, line):
         """
-        reduce non-ANSI chars if possible
+        normalize non-ANSI chars if possible
         """
         line = line.replace('“', '")')
         return line
 
+    @classmethod
+    def input_list_of_words(cls, words):
+        """
+        create list of words/phrases, maybe from argumment
+        ignores blank lines or lines starting with #
+        :param words: single word, list of words or file with words/phrases (one per line)
+        :return: list of words/phrases
+
+        """
+        if type(words) is list:
+            return words
+        #  read from file?
+        wordpath = Path(words)
+        if not wordpath.exists():
+            #  treat as single word/phrase
+            words = [words]
+        else:
+            with open(wordpath, "r") as f:
+                words = []
+                for line in f.readlines():
+                    line = line.strip()
+                    if line != "" and not line.startswith("#"):
+                        words.append(line)
+        return words
+
 
 class GithubDownloader:
     """Note: Github uses the old 'master' name but we have changed it to 'main'"""
@@ -899,6 +928,32 @@ class TextUtil:
             [c if c not in unwanted_chars else replacement for c in text])
         return text0
 
+    @classmethod
+    def convert_quoted_list_to_list(cls, quoted_list):
+        """
+        lists are often serialised as a strin of form '["A", "b", 1]'
+        see https://stackoverflow.com/questions/1894269/how-to-convert-string-representation-of-list-to-a-list
+        we have chosen the json method
+        Does NOT appear to parse "['a', 'b']"
+        :param quoted_list: list of form '[ "A","B","C" , " D"]'
+
+        """
+        if quoted_list is None:
+            return None
+        if type(quoted_list) is list:
+            return quoted_list
+        if type(quoted_list) is not str:
+            raise ValueError(f"not a quoted list: {quoted_list}")
+        if "'" in quoted_list:
+            # change all single quote to double
+            if '"' in quoted_list:
+                raise ValueError(f"list has both types of quotes, too difficult ATM")
+            # logger.debug(f"single quote {quoted_list}")
+            quoted_list = quoted_list.replace("'", '"')
+            # logger.debug(f"double quote {quoted_list}")
+        parsed_list = json.loads(quoted_list)
+        # logger.debug(f"parsed_list {len(parsed_list)} => {parsed_list[0]} ... {parsed_list}")
+        return parsed_list
 
 # sub/Super
 
