@@ -1,16 +1,14 @@
 # test util
 
 import csv
-import logging
-import re
 import shutil
-import sys
 import unittest
 from pathlib import Path
 
+from amilib.ami_util import AmiJson
 from amilib.file_lib import FileLib
 from amilib.util import EnhancedRegex
-from amilib.util import Util, GithubDownloader
+from amilib.util import Util, GithubDownloader, TextUtil
 from amilib.ami_args import ArgParseBuilder, AmiArgParser, AmiArgParseException
 from amilib.xml_lib import Templater
 
@@ -36,27 +34,6 @@ class Util0Test(AmiAnyTest):
     #     """restore args"""
     #     sys.argv = cls.sys_argv_save
 
-    @classmethod
-    @unittest.skip("not working properly, I think some tests change the args...")
-    # TODO fix args - some  tests change the args
-    def test_add_argstr(cls):
-        # this is a hack as normally there is only one element
-        # sys.argv = sys.argv[1:]
-        # assert sys.argv[1:] == []
-        cmd = "--help foo bar plinge"
-        Util.add_sys_argv_str(cmd)
-        assert sys.argv[1:] == ["--help", "foo", "bar", "plinge"]
-
-    @classmethod
-    @unittest.skip("not working properly")
-    # TODO fix args
-    def test_add_args(cls):
-        # this is a hack as normally there is only one element
-        sys.argv = sys.argv[1:]
-        # assert sys.argv[1:] == []
-        args = ["--help", "foox", "barx", "plingex"]
-        Util.add_sys_argv(args)
-        assert sys.argv[1:] == ["--help", "foox", "barx", "plingex"]
 
     @classmethod
     def test_copy_anything(cls):
@@ -172,26 +149,6 @@ class Util0Test(AmiAnyTest):
         url = "https://foo.bar/plugh/bloop.xml"
         assert Util.get_file_from_url(url) == "bloop.xml"
 
-    @unittest.skip("NYI")
-    def test_make_id_from_match_and_idgen(self):
-        """idgen is of the form <grouo>some text<group>
-        where groups correspond to named capture groups in regex
-        """
-        idgen = "12/CMA.34"
-        components = ["", ("decision", "\\d+"), "/", ("type", "CP|CMA|CMP"), "\\.", ("session", "\\d+"), ""]
-        enhanced_regex = EnhancedRegex(components=components)
-        id = enhanced_regex.make_id(idgen)
-        assert id == "12_CMA_34"
-
-    # ABANDONED
-    # def test_make_regex_with_capture_groups(self):
-    #     """idgen is of the form <grouo>some text<group>
-    #     where groups correspond to named capture groups in regex
-    #     """
-    #     enhanced_regex = EnhancedRegex()
-    #     components = ["", ("decision", "\d+"), "/", ("type", "CP|CMA|CMP"), "\.", ("session", "\d+"), ""]
-    #     regex = enhanced_regex.make_regex_with_capture_groups(components)
-    #     assert regex == '(?P<decision>\\d+)/(?P<type>CP|CMA|CMP)\\.(?P<session>\\d+)'
 
     def test_make_components_from_regex(self):
         """splits regex with capture groups into its components
@@ -213,25 +170,29 @@ class Util0Test(AmiAnyTest):
         if AmiAnyTest.IS_PMR:
             assert username == "pm286"
 
+    def test_parse_quoted_list(self):
+        quoted_list = '["a", "b"]'
+        parsed_list = TextUtil.convert_quoted_list_to_list(quoted_list)
+        assert type(parsed_list) is list
+        assert len(parsed_list) == 2
+        assert parsed_list[0] == "a"
+
+        quoted_list = '["a", "b", [1 ,2]]'
+        parsed_list = TextUtil.convert_quoted_list_to_list(quoted_list)
+        assert type(parsed_list) is list
+        assert len(parsed_list) == 3
+        assert parsed_list[2] == [1,2]
+
+    def test_parse_double_quoted_list(self):
+        """
+        single quotes fail in json, but we code round it
+        """
+        quoted_list = "['a', 'b']"
+        parsed_list = TextUtil.convert_quoted_list_to_list(quoted_list)
+        assert parsed_list == ["a", "b"]
 
-class GithubDownloaderTest(AmiAnyTest):
-    # def __init__(self):
-    #     pass
 
-    @unittest.skip("VERY LONG, DOWNLOADS")
-    def test_explore_main_page(self):
-        owner = "petermr"
-        repo = "CEVOpen"
-        downloader = GithubDownloader(owner=owner, repo=repo, max_level=1)
-        page = None
-        downloader.make_get_main_url()
-        logger.info(f"main page {downloader.main_url}")
-        url = downloader.main_url
-        if not url:
-            print(f"no page {owner}/{repo}")
-            return None
 
-        downloader.load_page(url, level=0)
 
 
 class AmiArgParserTest(AmiAnyTest):
@@ -310,3 +271,34 @@ class LoggingTest(AmiAnyTest):
 
         print(f"FINISHED")
 
+class AmiJsonTest(AmiAnyTest):
+    """
+    tricky JSON stuff (e.g. nested dicts)
+    """
+
+    def test_nested_dicts(self):
+        """
+        access values in nested dict by dot-separated strings
+        """
+        dikt = {
+            "key1" : "value1",
+            "key2" :
+                {
+                    "subkey1": "subvalue21",
+                    "subkey2": "subvalue22",
+
+                },
+            "key3":
+                {
+                    "subkey1":
+                        {
+                            "subsubkey1": "subsubvalue311",
+                            "subsubkey2": "subsubvalue312",
+                        },
+                }
+
+        }
+        assert "value1" == AmiJson.read_nested_dicts(dikt, "key1")
+        assert "subvalue21" == AmiJson.read_nested_dicts(dikt, "key2.subkey1")
+        assert "subsubvalue312" == AmiJson.read_nested_dicts(dikt, "key3.subkey1.subsubkey2")
+
